#define LINUX_20
#undef CONFIG_Z8530_ASYNC
#define RT_LOCK
#define RT_UNLOCK

#include <linux/module.h>
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/net.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/if_arp.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/tty.h>

#include <asm/dma.h>
#include <asm/io.h>

#ifdef LINUX_21
#include <asm/spinlock.h>
#endif

#include "z85230.h"


/*
 *	Provided port access methods. The Comtrol SV11 requires no delays
 *	between accesses and uses PC I/O. Some drivers may need a 5uS delay
 */

extern __inline__ int z8530_read_port(int p)
{
	u8 r=inb(Z8530_PORT_OF(p));
	if(p&Z8530_PORT_SLEEP)	/* gcc should figure this out efficiently ! */
		udelay(5);
	return r;
}

extern __inline__ void z8530_write_port(int p, u8 d)
{
	outb(d,Z8530_PORT_OF(p));
	if(p&Z8530_PORT_SLEEP)
		udelay(5);
}



static void z8530_rx_done(struct z8530_channel *c);
static void z8530_tx_done(struct z8530_channel *c);


/*
 *	Port accesses
 */
 
extern inline u8 read_zsreg(struct z8530_channel *c, u8 reg)
{
	u8 r;
	unsigned long flags;
	save_flags(flags);
	cli();
	if(reg)
		z8530_write_port(c->ctrlio, reg);
	r=z8530_read_port(c->ctrlio);
	restore_flags(flags);
	return r;
}

extern inline u8 read_zsdata(struct z8530_channel *c)
{
	u8 r;
	r=z8530_read_port(c->dataio);
	return r;
}

extern inline void write_zsreg(struct z8530_channel *c, u8 reg, u8 val)
{
	unsigned long flags;
	save_flags(flags);
	cli();
	if(reg)
		z8530_write_port(c->ctrlio, reg);
	z8530_write_port(c->ctrlio, val);
	restore_flags(flags);
}

extern inline void write_zsctrl(struct z8530_channel *c, u8 val)
{
	z8530_write_port(c->ctrlio, val);
}

extern inline void write_zsdata(struct z8530_channel *c, u8 val)
{
	z8530_write_port(c->dataio, val);
}

/*
 *	Register loading parameters for a dead port
 */
 
u8 z8530_dead_port[]=
{
	255
};


/*
 *	Register loading parameters for a BT kilostream
 */

u8 z8530_hdlc_kilostream[]=
{
	4,	SYNC_ENAB|SDLC|X1CLK,
	2,	0,	/* No vector */
	1,	0,
	3,	ENT_HM|RxCRC_ENAB|Rx8,
	5,	TxCRC_ENAB|RTS|TxENAB|Tx8|DTR,
	9,	0,		/* Disable interrupts */
	6,	0xFF,
	7,	FLAG,
	10,	ABUNDER|/*MARKIDLE|*/NRZ|CRCPS,
	11,	TCTRxCP,
	14,	DISDPLL,
	15,	DCDIE|SYNCIE|CTSIE|TxUIE|BRKIE,
	1,	EXT_INT_ENAB|TxINT_ENAB|INT_ALL_Rx,
	9,	NV|MIE|NORESET,
	255
};


u8 z8530_hdlc_kilostream_85230[]=
{
	4,	SYNC_ENAB|SDLC|X1CLK,
	2,	0,	/* No vector */
	1,	0,
	3,	ENT_HM|RxCRC_ENAB|Rx8,
	5,	TxCRC_ENAB|RTS|TxENAB|Tx8|DTR,
	9,	0,		/* Disable interrupts */
	6,	0xFF,
	7,	FLAG,
	10,	ABUNDER|/*MARKIDLE|*/NRZ|CRCPS,
	11,	TCTRxCP,
	14,	DISDPLL,
	15,	DCDIE|SYNCIE|CTSIE|TxUIE|BRKIE,
	1,	EXT_INT_ENAB|TxINT_ENAB|INT_ALL_Rx,
	9,	NV|MIE|NORESET,
	23,	3,		/* Extended mode AUTO TX and EOM*/
	
	255
};


u8 z8530_async_regs[]=
{
	9,	CHRA,		/* Reset CHR A */
	13,	0,		/* Baud */
	12,	1,		/* BaudLo */
	14,	BRENABL,	/* Baud generator on */
	11,	RCBR|TCBR,	/* Baud rate generator is the clock */
	5,	Tx8|TxENAB|RTS,	/* Transmit side up */
	4,	SB1|X16CLK,	/* Clock multiplier */
	3,	Rx8|RxENABLE,	/* Async RX on */
	255
};


/*
 *	Flush the FIFO
 */
 
static void z8530_flush_fifo(struct z8530_channel *c)
{
	read_zsreg(c, R1);
	read_zsreg(c, R1);
	read_zsreg(c, R1);
	read_zsreg(c, R1);
	if(c->dev->type==Z85230)
	{
		read_zsreg(c, R1);
		read_zsreg(c, R1);
		read_zsreg(c, R1);
		read_zsreg(c, R1);
	}
}	

/* Sets or clears DTR/RTS on the requested line */

static void z8530_rtsdtr(struct z8530_channel *c, int set)
{
	if (set)
		c->regs[5] |= (RTS | DTR);
	else
		c->regs[5] &= ~(RTS | DTR);
	write_zsreg(c, R5, c->regs[5]);
}

/*
 *	Receive handler. This is much like the async one but not quite the
 *	same or as complex
 *
 *	Note: Its intended that this handler can easily be seperated from
 *	the main code to run realtime. That'll be needed for some machines
 *	(eg to ever clock 64kbits on a sparc ;)).
 *
 *	The RT_LOCK macros don't do anything now. Keep the code covered
 *	by them as short as possible in all circumstances - clocks cost
 *	baud. The interrupt handler is assumed to be atomic w.r.t. to
 *	other code - this is true in the RT case too.
 *
 *	We only cover the sync cases for this. If you want 2Mbit async
 *	do it yourself.
 */
 
static void z8530_rx(struct z8530_channel *c)
{
	u8 ch,stat;
	 
	while(1)
	{
		/* FIFO empty ? */
		if(!(read_zsreg(c, R0)&1))
			break;
		ch=read_zsdata(c);
		stat=read_zsreg(c, R1);
	
		/*
		 *	Overrun ?
		 */
		if(c->count < c->max)
		{
			*c->dptr++=ch;
			c->count++;
		}

		if(stat&END_FR)
		{
		
			/*
			 *	Error ?
			 */
			if(stat&(Rx_OVR|CRC_ERR))
			{
				/* Rewind the buffer and return */
				if(c->skb)
					c->dptr=c->skb->data;
				c->count=0;
				if(stat&Rx_OVR)
				{
					c->rx_overrun++;
				}
				if(stat&CRC_ERR)
				{
					c->rx_crc_err++;
				}
				/* Shove the frame upstream */
			}
			else
			{
				z8530_rx_done(c);
				write_zsctrl(c, RES_Rx_CRC);
			}
		}
	}
	/*
	 *	Clear irq
	 */
	write_zsctrl(c, ERR_RES);
	write_zsctrl(c, RES_H_IUS);
}


/*
 *	Z8530 transmit interrupt handler
 */
 
static void z8530_tx(struct z8530_channel *c)
{
	while(c->txcount)
	{
		/* FIFO full ? */
		if(!(read_zsreg(c, R0)&4))
			break;
		c->txcount--;
		/*
		 *	Shovel out the byte
		 */
		write_zsreg(c, R8, *c->tx_ptr++);
		write_zsctrl(c, RES_H_IUS);
		/* We are about to underflow */
		if(c->txcount==0)
		{
			write_zsctrl(c, RES_EOM_L);
			write_zsreg(c, R10, c->regs[10]&~ABUNDER);
		}
		return;
	}
	
	/*
	 *	End of frame TX - fire another one
	 */
	 
	write_zsctrl(c, RES_Tx_P);

	z8530_tx_done(c);	 
//	write_zsreg(c, R8, *c->tx_ptr++);
	write_zsctrl(c, RES_H_IUS);
}

static void z8530_status(struct z8530_channel *chan)
{
	u8 status=read_zsreg(chan, R0);
	u8 altered=chan->status^status;
	
	chan->status=status;
	
	if(status&TxEOM)
	{
		write_zsctrl(chan, ERR_RES);
		z8530_tx_done(chan);
	}
		
	if(altered&DCD)
	{
		if(status&DCD)
		{
			printk(KERN_INFO "DCD raised\n");
			write_zsreg(chan, R3, chan->regs[3]|RxENABLE);
		}
		else
		{
			printk(KERN_INFO "DCD lost\n");
			write_zsreg(chan, R3, chan->regs[3]&~RxENABLE);
			z8530_flush_fifo(chan);
		}
		
	}	
	write_zsctrl(chan, RES_EXT_INT);
	write_zsctrl(chan, RES_H_IUS);
}

struct z8530_irqhandler z8530_sync=
{
	z8530_rx,
	z8530_tx,
	z8530_status
};

/*
 *	Non bus mastering DMA interfaces for the Z8x30 devices. This
 *	is really pretty PC specific.
 */
 
void z8530_dma_rx(struct z8530_channel *chan)
{
	if(chan->rxdma_on)
	{
		/* Special condition check only */
		u8 status;
		
		read_zsreg(chan, R7);
		read_zsreg(chan, R6);
		status=read_zsreg(chan, R1);
		if(status&END_FR)
		{
			z8530_rx_done(chan);	/* Fire up the next one */
		}		
		write_zsctrl(chan, ERR_RES);
		write_zsctrl(chan, RES_H_IUS);
	}
	else
	{
		/* DMA is off right now, drain the slow way */
		z8530_rx(chan);
	}	
}

void z8530_dma_tx(struct z8530_channel *chan)
{
	if(!chan->dma_tx)
	{
		printk("z85230: Hey who turned the DMA off\n");
		z8530_tx(chan);
		return;
	}
	printk("z85230: received TX irq in DMA mode.\n");	
	write_zsctrl(chan, RES_H_IUS);
}

void z8530_dma_status(struct z8530_channel *chan)
{
	u8 status=read_zsreg(chan, R0);
	u8 altered=chan->status^status;
	
	chan->status=status;

	if(chan->dma_tx)
	{
		if(status&TxEOM)
		{
			/* Transmit underrun */
			disable_dma(chan->txdma);
			clear_dma_ff(chan->txdma);	
			chan->txdma_on=0;
			z8530_tx_done(chan);
		}
	}
	if(altered&DCD)
	{
		if(status&DCD)
		{
			printk(KERN_INFO "DCD raised\n");
			write_zsreg(chan, R3, chan->regs[3]|RxENABLE);
		}
		else
		{
			printk(KERN_INFO "DCD lost\n");
			write_zsreg(chan, R3, chan->regs[3]&~RxENABLE);
			z8530_flush_fifo(chan);
		}
	}	
	write_zsctrl(chan, RES_EXT_INT);
	write_zsctrl(chan, RES_H_IUS);
}

struct z8530_irqhandler z8530_dma_sync=
{
	z8530_dma_rx,
	z8530_dma_tx,
	z8530_dma_status
};


struct z8530_irqhandler z8530_txdma_sync=
{
	z8530_rx,
	z8530_dma_tx,
	z8530_dma_status
};

/*
 *	Asynchronous mode 8530 receiver
 */
 
extern inline void z8530_sched_event(struct z8530_channel *c,
				  int event)
{
	c->event |= 1 << event;
//	queue_task(&c->tqueue, &tq_serial);
//	mark_bh(SERIAL_BH);
}
 
 
static void z8530_rx_async(struct z8530_channel *c)
{
	struct tty_struct *tty = c->tty;
	unsigned char ch, stat,flag;
	
	while((read_zsreg(c,R0)&Rx_CH_AV)!=0)
	{
		stat=read_zsreg(c,R1);
		ch=read_zsdata(c);
		if(!tty)
			continue;
//		tty_flip_buffer_push(tty);
		if(tty->flip.count>=TTY_FLIPBUF_SIZE)
			continue;
		tty->flip.count++;
		if(stat&Rx_OVR)
		{
			flag = TTY_OVERRUN;
			write_zsreg(c, R0, ERR_RES);
		}
		else if(stat & CRC_ERR) 
		{
			flag = TTY_FRAME;
		}
		else if(stat & PAR_ERR)
		{
			flag = TTY_PARITY;
			write_zsreg(c, R0, ERR_RES);
		}
		else
			flag=0;
			
		*tty->flip.flag_buf_ptr++ = flag;
		*tty->flip.char_buf_ptr++ = ch;
	}
}

static void z8530_tx_async(struct z8530_channel *c)
{
	if((read_zsreg(c, R0)&Tx_BUF_EMP)==0)
		return;
	c->tx_active=0;
	if(c->x_char)
	{
		write_zsdata(c, c->x_char);
		c->x_char=0;
		c->tx_active=1;
		return;
	}
	if((c->xmit_cnt <= 0) || c->tty->stopped || c->tx_stopped)
	{
		write_zsreg(c, R0, RES_Tx_P);
		return;
	}
	write_zsdata(c, c->xmit_buf[c->xmit_tail++]);
	c->xmit_tail = c->xmit_tail & (SERIAL_XMIT_SIZE-1);
	c->xmit_cnt--;
	c->tx_active = 1;
	if(c->xmit_cnt < WAKEUP_CHARS)
		z8530_sched_event(c, RS_EVENT_WRITE_WAKEUP);
}




static void z8530_status_async(struct z8530_channel *c)
{
	unsigned char status;

	/* Get status from Read Register 0 */
	status = read_zsreg(c, R0);

	/* Check for DCD transitions */
	if (((status ^ c->status) & DCD) != 0
	    && c->tty && !C_CLOCAL(c->tty)) {
		if (status & DCD) {
			wake_up_interruptible(&c->open_wait);
		} else if (!(c->flags & ZILOG_CALLOUT_ACTIVE)) {
			if (c->tty)
				tty_hangup(c->tty);
		}
	}

	/* Check for CTS transitions */
	if (c->tty && C_CRTSCTS(c->tty)) {
		/*
		 * For some reason, on the Power Macintosh,
		 * it seems that the CTS bit is 1 when CTS is
		 * *negated* and 0 when it is asserted.
		 * The DCD bit doesn't seem to be inverted
		 * like this.
		 */
		if ((status & CTS) == 0) {
			if (c->tx_stopped) {
				c->tx_stopped = 0;
				if (!c->tx_active)
					z8530_tx_async(c);
			}
		} else {
			c->tx_stopped = 1;
		}
	}

	/* Clear status condition... */
	write_zsreg(c, R0, RES_EXT_INT);
	c->status = status;
}

struct z8530_irqhandler z8530_async=
{
	z8530_rx_async,
	z8530_tx_async,
	z8530_status_async
};

/*
 *	Interrupt vectors for a Z8530 that is in 'parked' mode.
 */


static void z8530_rx_clear(struct z8530_channel *c)
{
	/*
	 *	Data and status bytes
	 */
	u8 stat;

	read_zsdata(c);
	stat=read_zsreg(c, R1);
	
	if(stat&END_FR)
		write_zsctrl(c, RES_Rx_CRC);
	/*
	 *	Clear irq
	 */
	write_zsctrl(c, ERR_RES);
	write_zsctrl(c, RES_H_IUS);
}

static void z8530_tx_clear(struct z8530_channel *c)
{
	write_zsctrl(c, RES_Tx_P);
	write_zsctrl(c, RES_H_IUS);
}

static void z8530_status_clear(struct z8530_channel *chan)
{
	u8 status=read_zsreg(chan, R0);
	if(status&TxEOM)
		write_zsctrl(chan, ERR_RES);
	write_zsctrl(chan, RES_EXT_INT);
	write_zsctrl(chan, RES_H_IUS);
}

struct z8530_irqhandler z8530_nop=
{
	z8530_rx_clear,
	z8530_tx_clear,
	z8530_status_clear
};


/*
 *	A Z85[2]30 device has stuck its hand in the air for attention
 */

 
void z8530_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	struct z8530_dev *dev=dev_id;
	u8 intr;
	static volatile int locker=0;
	int work=0;
	
	if(locker)
	{
		printk(KERN_ERR "IRQ re-enter\n");
		return;
	}
	locker=1;
	
	while(++work<5000)
	{
		struct z8530_irqhandler *irqs=dev->chanA.irqs;
		
		intr = read_zsreg(&dev->chanA, R3);
		if(!(intr & (CHARxIP|CHATxIP|CHAEXT|CHBRxIP|CHBTxIP|CHBEXT)))
			break;
	
		/* This holds the IRQ status. On the 8530 you must read it from chan 
		   A even though it applies to the whole chip */
		
		/* Now walk the chip and see what it is wanting - it may be
		   an IRQ for someone else remember */
		   
		if(intr & (CHARxIP|CHATxIP|CHAEXT))
		{
			if(intr&CHARxIP)
				irqs->rx(&dev->chanA);
			if(intr&CHATxIP)
				irqs->tx(&dev->chanA);
			if(intr&CHAEXT)
				irqs->status(&dev->chanA);
		}

		irqs=dev->chanB.irqs;

		if(intr & (CHBRxIP|CHBTxIP|CHBEXT))
		{
			if(intr&CHBRxIP)
				irqs->rx(&dev->chanB);
			if(intr&CHBTxIP)
				irqs->tx(&dev->chanB);
			if(intr&CHBEXT)
				irqs->status(&dev->chanB);
		}
	}
	if(work==200)
		printk(KERN_WARNING "%s: interrupt jammed - abort(0x%X)!\n", dev->name, intr);
	/* Ok all done */
	locker=0;
}



static char reg_init[16]=
{
	0,0,0,0,
	0,0,0,0,
	0,0,0,0,
	0x55,0,0,0
};


int z8530_sync_open(struct device *dev, struct z8530_channel *c)
{
	c->sync = 1;
	c->mtu = dev->mtu;
	c->count = 0;
	c->skb = NULL;
	c->skb2 = NULL;
	c->irqs = &z8530_sync;
	/* This loads the double buffer up */
	z8530_rx_done(c);	/* Load the frame ring */
	z8530_rx_done(c);	/* Load the backup frame */
	z8530_rtsdtr(c,1);
	c->dma_tx = 0;
	c->regs[R1]|= TxINT_ENAB;
	write_zsreg(c, R1, c->regs[R1]);
	write_zsreg(c, R3, c->regs[R3]|RxENABLE);
	return 0;
}


int z8530_sync_close(struct device *dev, struct z8530_channel *c)
{
	u8 chk;
	c->irqs = &z8530_nop;
	c->max = 0;
	c->sync = 0;
	
	chk=read_zsreg(c,R0);
	write_zsreg(c, R3, c->regs[R3]);
	z8530_rtsdtr(c,0);
	return 0;
}

int z8530_sync_dma_open(struct device *dev, struct z8530_channel *c)
{
	c->sync = 1;
	c->mtu = dev->mtu;
	c->count = 0;

	c->skb = NULL;
	c->skb2 = NULL;
	/*
	 *	Load the DMA interfaces up
	 */
	c->rxdma_on = 0;
	c->txdma_on = 0;
	
	/*
	 *	Allocate the DMA flip buffers
	 */
	 
	c->rx_buf[0]=kmalloc(c->mtu+128, GFP_KERNEL|GFP_DMA);
	if(c->rx_buf[0]==NULL)
		return -ENOBUFS;
	c->rx_buf[1]=kmalloc(c->mtu+128, GFP_KERNEL|GFP_DMA);
	if(c->rx_buf[1]==NULL)
	{
		kfree(c->rx_buf[0]);
		c->rx_buf[0]=NULL;
		return -ENOBUFS;
	}
	
	c->tx_dma_buf[0]=kmalloc(c->mtu+128, GFP_KERNEL|GFP_DMA);
	if(c->tx_dma_buf[0]==NULL)
	{
		kfree(c->rx_buf[0]);
		kfree(c->rx_buf[1]);
		c->rx_buf[0]=NULL;
		return -ENOBUFS;
	}
	c->tx_dma_buf[1]=kmalloc(c->mtu+128, GFP_KERNEL|GFP_DMA);
	if(c->tx_dma_buf[1]==NULL)
	{
		kfree(c->tx_dma_buf[0]);
		kfree(c->rx_buf[0]);
		kfree(c->rx_buf[1]);
		c->rx_buf[0]=NULL;
		c->rx_buf[1]=NULL;
		c->tx_dma_buf[0]=NULL;
		return -ENOBUFS;
	}
	c->tx_dma_used=0;
	c->dma_tx = 1;
	c->dma_num=0;
	c->dma_ready=1;
	
	/*
	 *	Enable DMA control mode
	 */
	 
	/*
	 *	TX DMA via DIR/REQ
	 */
	 
	c->regs[R14]|= DTRREQ;
	write_zsreg(c, R14, c->regs[R14]);     

	/*
	 *	RX DMA via W/Req
	 */	 

	c->regs[R1]|= WT_FN_RDYFN;
	c->regs[R1]|= WT_RDY_RT;
	c->regs[R1]|= INT_ERR_Rx;
	c->regs[R1]&= ~TxINT_ENAB;
	
	write_zsreg(c, R1, c->regs[R1]);
	c->regs[R1]|= WT_RDY_ENAB;
	write_zsreg(c, R1, c->regs[R1]);            
	
	
	/*
	 *	DMA interrupts
	 */
	 
	/*
	 *	Set up the DMA configuration
	 */	
	 
	disable_dma(c->rxdma);
	clear_dma_ff(c->rxdma);
	set_dma_mode(c->rxdma, DMA_MODE_READ|0x10);
	set_dma_addr(c->rxdma, virt_to_bus(c->rx_buf[0]));
	set_dma_count(c->rxdma, c->mtu);
	enable_dma(c->rxdma);

	disable_dma(c->txdma);
	clear_dma_ff(c->txdma);
	set_dma_mode(c->txdma, DMA_MODE_WRITE);
	disable_dma(c->txdma);
	
	/*
	 *	Select the DMA interrupt handlers
	 */

	c->rxdma_on = 1;
	c->txdma_on = 1;
	c->tx_dma_used = 1;
	 
	c->irqs = &z8530_dma_sync;
	z8530_rtsdtr(c,1);
	write_zsreg(c, R3, c->regs[R3]|RxENABLE);
	return 0;
}


int z8530_sync_dma_close(struct device *dev, struct z8530_channel *c)
{
	u8 chk;
	c->irqs = &z8530_nop;
	c->max = 0;
	c->sync = 0;
	
	/*
	 *	Disable the PC DMA channels
	 */
	 
	disable_dma(c->rxdma);
	clear_dma_ff(c->rxdma);
	c->rxdma_on = 0;
	disable_dma(c->txdma);
	clear_dma_ff(c->txdma);
	c->txdma_on = 0;
	c->tx_dma_used = 0;

	/*
	 *	Disable DMA control mode
	 */
	 
	c->regs[R1]&= ~WT_RDY_ENAB;
	write_zsreg(c, R1, c->regs[R1]);            
	c->regs[R1]&= ~(WT_RDY_RT|WT_FN_RDYFN|INT_ERR_Rx);
	c->regs[R1]|= INT_ALL_Rx;
	write_zsreg(c, R1, c->regs[R1]);
	c->regs[R14]&= ~DTRREQ;
	write_zsreg(c, R14, c->regs[R14]);   
	
	if(c->rx_buf[0])
	{
		kfree(c->rx_buf[0]);
		c->rx_buf[0]=NULL;
	}
	if(c->rx_buf[1])
	{
		kfree(c->rx_buf[1]);
		c->rx_buf[1]=NULL;
	}
	if(c->tx_dma_buf[0])
	{
		kfree(c->tx_dma_buf[0]);
		c->tx_dma_buf[0]=NULL;
	}
	if(c->tx_dma_buf[1])
	{
		kfree(c->tx_dma_buf[1]);
		c->tx_dma_buf[1]=NULL;
	}
	chk=read_zsreg(c,R0);
	write_zsreg(c, R3, c->regs[R3]);
	z8530_rtsdtr(c,0);
	return 0;
}

int z8530_sync_txdma_open(struct device *dev, struct z8530_channel *c)
{
	c->sync = 1;
	c->mtu = dev->mtu;
	c->count = 0;
	c->skb = NULL;
	c->skb2 = NULL;
	
	/*
	 *	Load the PIO receive ring
	 */

	z8530_rx_done(c);
	z8530_rx_done(c);

	/*
	 *	Load the DMA interfaces up
	 */
	c->rxdma_on = 0;
	c->txdma_on = 0;
	
	c->tx_dma_buf[0]=kmalloc(c->mtu+128, GFP_KERNEL|GFP_DMA);
	if(c->tx_dma_buf[0]==NULL)
	{
		kfree(c->rx_buf[0]);
		kfree(c->rx_buf[1]);
		c->rx_buf[0]=NULL;
		return -ENOBUFS;
	}
	c->tx_dma_buf[1]=kmalloc(c->mtu+128, GFP_KERNEL|GFP_DMA);
	if(c->tx_dma_buf[1]==NULL)
	{
		kfree(c->tx_dma_buf[0]);
		kfree(c->rx_buf[0]);
		kfree(c->rx_buf[1]);
		c->rx_buf[0]=NULL;
		c->rx_buf[1]=NULL;
		c->tx_dma_buf[0]=NULL;
		return -ENOBUFS;
	}
	c->tx_dma_used=0;
	c->dma_num=0;
	c->dma_ready=1;
	c->dma_tx = 1;
	
	/*
	 *	Enable DMA control mode
	 */
	 
	/*
	 *	TX DMA via DIR/REQ
	 */
	 
	c->regs[R14]|= DTRREQ;
	write_zsreg(c, R14, c->regs[R14]);     

	c->regs[R1]&= ~TxINT_ENAB;
	write_zsreg(c, R1, c->regs[R1]);

	
	/*
	 *	Set up the DMA configuration
	 */	
	 
	disable_dma(c->txdma);
	clear_dma_ff(c->txdma);
	set_dma_mode(c->txdma, DMA_MODE_WRITE);
	disable_dma(c->txdma);
	
	/*
	 *	Select the DMA interrupt handlers
	 */

	c->rxdma_on = 0;
	c->txdma_on = 1;
	c->tx_dma_used = 1;
	 
	c->irqs = &z8530_txdma_sync;
	z8530_rtsdtr(c,1);
	write_zsreg(c, R3, c->regs[R3]|RxENABLE);
	return 0;
}


int z8530_sync_txdma_close(struct device *dev, struct z8530_channel *c)
{
	u8 chk;
	c->irqs = &z8530_nop;
	c->max = 0;
	c->sync = 0;
	
	/*
	 *	Disable the PC DMA channels
	 */
	 
	disable_dma(c->txdma);
	clear_dma_ff(c->txdma);
	c->txdma_on = 0;
	c->tx_dma_used = 0;

	/*
	 *	Disable DMA control mode
	 */
	 
	c->regs[R1]&= ~WT_RDY_ENAB;
	write_zsreg(c, R1, c->regs[R1]);            
	c->regs[R1]&= ~(WT_RDY_RT|WT_FN_RDYFN|INT_ERR_Rx);
	c->regs[R1]|= INT_ALL_Rx;
	write_zsreg(c, R1, c->regs[R1]);
	c->regs[R14]&= ~DTRREQ;
	write_zsreg(c, R14, c->regs[R14]);   
	
	if(c->tx_dma_buf[0])
	{
		kfree(c->tx_dma_buf[0]);
		c->tx_dma_buf[0]=NULL;
	}
	if(c->tx_dma_buf[1])
	{
		kfree(c->tx_dma_buf[1]);
		c->tx_dma_buf[1]=NULL;
	}
	chk=read_zsreg(c,R0);
	write_zsreg(c, R3, c->regs[R3]);
	z8530_rtsdtr(c,0);
	return 0;
}

/************************* BEGIN ASYNCHRONOUS SUPPORT *********************/

#ifdef CONFIG_Z8530_ASYNC

/*
 *	Asynchronous back end code. This is _way_ more complex than the sync
 *	mode stuff! This is mixed from the Sparc, and Mac 8530 serial drivers.
 *
 *	Async mode requires the interrupt is NOT running in a real time mode.
 */

static void z8530_async_stop(struct tty_struct *tty)
{
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;

	if (serial_paranoia_check(c, tty->device, "rs_stop"))
		return;
#if 0
	save_flags(flags); cli();
	if (c->regs[5] & TxENAB) {
		c->regs[5] &= ~TxENAB;
		c->pendregs[5] &= ~TxENAB;
		write_zsreg(c, R5, c->regs[5]);
	}
	restore_flags(flags);
#endif
}

static void z8530_async_start(struct tty_struct *tty)
{
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;
	unsigned long flags;
	
	if (serial_paranoia_check(c, tty->device, "rs_start"))
		return;
	
	save_flags(flags); cli();
#if 0
	if (c->xmit_cnt && c->xmit_buf && !(c->regs[5] & TxENAB)) {
		c->regs[5] |= TxENAB;
		c->pendregs[5] = c->regs[5];
		write_zsreg(c, 5, c->regs[5]);
	}
#else
	if (c->xmit_cnt && c->xmit_buf && !c->tx_active) {
		z8530_tx_async(c);
	}
#endif
	restore_flags(flags);
}

/*
 * This routine is used to handle the "bottom half" processing for the
 * serial driver, known also the "software interrupt" processing.
 * This processing is done at the kernel interrupt level, after the
 * rs_interrupt() has returned, BUT WITH INTERRUPTS TURNED ON.  This
 * is where time-consuming activities which can not be done in the
 * interrupt driver proper are done; the interrupt driver schedules
 * them using z8530_sched_event(), and they get done here.
 */

static void z8530_do_bh(void)
{
	run_task_queue(&tq_serial);
}

static void z8530_do_softint(void *private_)
{
	struct z8530_channel	*c = (struct z8530_channel *) private_;
	struct tty_struct	*tty;
	
	tty = c->tty;
	if (!tty)
		return;

	if (test_and_clear_bit(RS_EVENT_WRITE_WAKEUP, &c->event)) {
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
		    tty->ldisc.write_wakeup)
			(tty->ldisc.write_wakeup)(tty);
		wake_up_interruptible(&tty->write_wait);
	}
}

static int z8530_async_startup(struct z8530_channel *c)
{
	unsigned long flags;

	if (c->flags & ZILOG_INITIALIZED)
		return 0;

	if (!c->xmit_buf) {
		c->xmit_buf = (unsigned char *) get_free_page(GFP_KERNEL);
		if (!c->xmit_buf)
			return -ENOMEM;
	}

	save_flags(flags); cli();

#ifdef SERIAL_DEBUG_OPEN
	printk("starting up scc tty %d (irq %d)...", c->line, c->irq);
#endif

	/*
	 * Clear the receive FIFO.
	 */

	z8530_flush_fifo(c);
	c->xmit_fifo_size = 1;

	/*
	 * Clear the interrupt registers.
	 */

	write_zsreg(c, R0, ERR_RES);
	write_zsreg(c, R0, RES_H_IUS);

	/*
	 * Turn on RTS and DTR.
	 */
	 
	z8530_rtsdtr(c, 1);

	/*
	 * Finally, enable sequencing and interrupts
	 */

	c->regs[1] = (c->regs[1] & ~0x18) | (EXT_INT_ENAB | INT_ALL_Rx | TxINT_ENAB);
	c->pendregs[1] = c->regs[1];
	c->regs[3] |= (RxENABLE | Rx8);
	c->pendregs[3] = c->regs[3];
	c->regs[5] |= (TxENAB | Tx8);
	c->pendregs[5] = c->regs[5];
	c->regs[9] |= (NV | MIE);
	c->pendregs[9] = c->regs[9];
	write_zsreg(c, R3, c->regs[3]);
	write_zsreg(c, R5, c->regs[5]);
	write_zsreg(c, R9, c->regs[9]);

	if (c->tty)
		clear_bit(TTY_IO_ERROR, &c->tty->flags);
	c->xmit_cnt = c->xmit_head = c->xmit_tail = 0;

	/*
	 * Set the speed of the serial port
	 */
	
	z8530_change_async_speed(c);

	/* Save the current value of RR0 */
	c->status = read_zsreg(c, R0);

	c->flags |= ZILOG_INITIALIZED;
	restore_flags(flags);
	return 0;
}

/*
 * This routine will shutdown a serial port; interrupts are disabled, and
 * DTR is dropped if the hangup on close termio flag is on.
 */

static void z8530_async_shutdown(struct z8530_channel *c)
{
	unsigned long flags;

	if (!(c->flags & ZILOG_INITIALIZED))
		return;

#ifdef SERIAL_DEBUG_OPEN
	printk("Shutting down serial port %d (irq %d)....", c->line,
	       c->irq);
#endif
	
	save_flags(flags); cli(); /* Disable interrupts */
	
	if (c->xmit_buf) {
		free_page((unsigned long) c->xmit_buf);
		c->xmit_buf = 0;
	}

	c->pendregs[1] = c->regs[1] = 0;
	write_zsreg(c, R1, 0);	/* no interrupts */

	c->regs[3] &= ~RxENABLE;
	c->pendregs[3] = c->regs[3];
	write_zsreg(c, R3, c->regs[3]);

	c->regs[5] &= ~TxENAB;
	if (!c->tty || C_HUPCL(c->tty))
		c->regs[5] &= ~(DTR | RTS);
	c->pendregs[5] = c->regs[5];
	write_zsreg(c, R5, c->regs[5]);

	if (c->tty)
		set_bit(TTY_IO_ERROR, &c->tty->flags);

	c->flags &= ~ZILOG_INITIALIZED;
	restore_flags(flags);
}

/*
 * This routine is called to set the UART divisor registers to match
 * the specified baud rate for a serial port.
 */

static void z8530_change_speed(struct z8530_channel *c)
{
	unsigned short port;
	unsigned cflag;
	int	i;
	int	brg;
	unsigned long flags;

	if (!c->tty || !c->tty->termios)
		return;
	cflag = c->tty->termios->c_cflag;
//	if (!(port = c->port))
//		return;
	i = cflag & CBAUD;

	save_flags(flags); cli();
	c->zs_baud = baud_table[i];
	c->clk_divisor = 16;

	switch (c->zs_baud) {
	case ZS_CLOCK/16:	/* 230400 */
		c->regs[4] = X16CLK;
		c->regs[11] = 0;
		break;
	case ZS_CLOCK/32:	/* 115200 */
		c->regs[4] = X32CLK;
		c->regs[11] = 0;
		break;
	default:
		c->regs[4] = X16CLK;
		c->regs[11] = TCBR | RCBR;
		brg = BPS_TO_BRG(c->zs_baud, ZS_CLOCK/c->clk_divisor);
		c->regs[12] = (brg & 255);
		c->regs[13] = ((brg >> 8) & 255);
		c->regs[14] = BRENABL;
	}

	/* byte size and parity */
	c->regs[3] &= ~RxNBITS_MASK;
	c->regs[5] &= ~TxNBITS_MASK;
	switch (cflag & CSIZE) 
	{
		case CS5:
			c->regs[3] |= Rx5;
			c->regs[5] |= Tx5;
			break;
		case CS6:
			c->regs[3] |= Rx6;
			c->regs[5] |= Tx6;
			break;
		case CS7:
			c->regs[3] |= Rx7;
			c->regs[5] |= Tx7;
			break;
		case CS8:
		default: /* defaults to 8 bits */
			c->regs[3] |= Rx8;
			c->regs[5] |= Tx8;
			break;
	}
	c->pendregs[3] = c->regs[3];
	c->pendregs[5] = c->regs[5];

	c->regs[4] &= ~(SB_MASK | PAR_ENA | PAR_EVEN);
	if (cflag & CSTOPB) {
		c->regs[4] |= SB2;
	} else {
		c->regs[4] |= SB1;
	}
	if (cflag & PARENB) {
		c->regs[4] |= PAR_ENA;
	}
	if (!(cflag & PARODD)) {
		c->regs[4] |= PAR_EVEN;
	}
	c->pendregs[4] = c->regs[4];

	if (!(cflag & CLOCAL)) {
		if (!(c->regs[15] & DCDIE))
			c->status = read_zsreg(c, R0);
		c->regs[15] |= DCDIE;
	} else
		c->regs[15] &= ~DCDIE;
	if (cflag & CRTSCTS) {
		c->regs[15] |= CTSIE;
		if ((read_zsreg(c, R0) & CTS) != 0)
			c->tx_stopped = 1;
	} else {
		c->regs[15] &= ~CTSIE;
		c->tx_stopped = 0;
	}
	c->pendregs[15] = c->regs[15];

	/* Load up the new values */
	z8530_load_zsregs(c, c->regs);

	restore_flags(flags);
}

static void z8530_async_flush_chars(struct tty_struct *tty)
{
	struct z8530_channel *c= (struct z8530_channel *)tty->driver_data;
	unsigned long flags;

	if (serial_paranoia_check(c, tty->device, "rs_flush_chars"))
		return;

	if (c->xmit_cnt <= 0 || tty->stopped || c->tx_stopped ||
	    !c->xmit_buf)
		return;

	/* Enable transmitter */
	save_flags(flags); cli();
	z8530_tx_async(c);
	restore_flags(flags);
}

static int z8530_async_write(struct tty_struct * tty, int from_user,
		    const unsigned char *buf, int count)
{
	int	ch, total = 0;
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;
	unsigned long flags;

	if (serial_paranoia_check(c, tty->device, "rs_write"))
		return 0;

	if (!tty || !c->xmit_buf)
		return 0;

	save_flags(flags);
	while (1) {
		cli();		
		ch = MIN(count, MIN(SERIAL_XMIT_SIZE - c->xmit_cnt - 1,
				   SERIAL_XMIT_SIZE - c->xmit_head));
		if (ch <= 0)
			break;

		if (from_user) {
			down(&tmp_buf_sem);
			copy_from_user(tmp_buf, buf, ch);
			ch = MIN(ch, MIN(SERIAL_XMIT_SIZE - c->xmit_cnt - 1,
				       SERIAL_XMIT_SIZE - c->xmit_head));
			memcpy(c->xmit_buf + c->xmit_head, tmp_buf, ch);
			up(&tmp_buf_sem);
		} else
			memcpy(c->xmit_buf + c->xmit_head, buf, ch);
		c->xmit_head = (c->xmit_head + ch) & (SERIAL_XMIT_SIZE-1);
		c->xmit_cnt += ch;
		restore_flags(flags);
		buf += ch;
		count -= ch;
		total += ch;
	}
	if (c->xmit_cnt && !tty->stopped && !c->tx_stopped
	    && !c->tx_active)
		z8530_tx_async(c);
	restore_flags(flags);
	return total;
}

static int z8530_async_write_room(struct tty_struct *tty)
{
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;
	int	ret;
				
	if (serial_paranoia_check(c, tty->device, "rs_write_room"))
		return 0;
	ret = SERIAL_XMIT_SIZE - c->xmit_cnt - 1;
	if (ret < 0)
		ret = 0;
	return ret;
}

static int z8530_chars_in_buffer(struct tty_struct *tty)
{
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;
				
	if (serial_paranoia_check(c, tty->device, "rs_chars_in_buffer"))
		return 0;
	return c->xmit_cnt;
}

static void z8530_async_flush_buffer(struct tty_struct *tty)
{
	struct z8530_channel *c= (struct z8530_channel *)tty->driver_data;
				
	if (serial_paranoia_check(c, tty->device, "rs_flush_buffer"))
		return;
	cli();
	c->xmit_cnt = c->xmit_head = c->xmit_tail = 0;
	sti();
	wake_up_interruptible(&tty->write_wait);
	if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
	    tty->ldisc.write_wakeup)
		(tty->ldisc.write_wakeup)(tty);
}

/*
 * ------------------------------------------------------------
 * z8530_async_throttle()
 * 
 * This routine is called by the upper-layer tty layer to signal that
 * incoming characters should be throttled.
 * ------------------------------------------------------------
 */
static void z8530_async_throttle(struct tty_struct * tty)
{
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;
	unsigned long flags;
#ifdef SERIAL_DEBUG_THROTTLE
	char	buf[64];
	
	printk("throttle %s: %d....\n", _tty_name(tty, buf),
	       tty->ldisc.chars_in_buffer(tty));
#endif

	if (serial_paranoia_check(c, tty->device, "z8530_async_throttle"))
		return;
	
	if (I_IXOFF(tty)) {
		save_flags(flags); cli();
		c->x_char = STOP_CHAR(tty);
		if (!c->tx_active)
			z8530_tx_async(c);
		restore_flags(flags);
	}

	if (C_CRTSCTS(tty)) {
		/*
		 * Here we want to turn off the RTS line.  On Macintoshes,
		 * we only get the DTR line, which goes to both DTR and
		 * RTS on the modem.  RTS doesn't go out to the serial
		 * port socket.  So you should make sure your modem is
		 * set to ignore DTR if you're using CRTSCTS.
		 */
		save_flags(flags); cli();
		c->regs[5] &= ~(DTR | RTS);
		c->pendregs[5] &= ~(DTR | RTS);
		write_zsreg(c, R5, c->regs[5]);
		restore_flags(flags);
	}
}

static void z8530_async_unthrottle(struct tty_struct * tty)
{
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;
	unsigned long flags;
#ifdef SERIAL_DEBUG_THROTTLE
	char	buf[64];
	
	printk("unthrottle %s: %d....\n", _tty_name(tty, buf),
	       tty->ldisc.chars_in_buffer(tty));
#endif

	if (serial_paranoia_check(c, tty->device, "z8530_async_unthrottle"))
		return;
	
	if (I_IXOFF(tty)) {
		save_flags(flags); cli();
		if (c->x_char)
			c->x_char = 0;
		else {
			c->x_char = START_CHAR(tty);
			if (!c->tx_active)
				z8530_tx_async(c);
		}
		restore_flags(flags);
	}

	if (C_CRTSCTS(tty)) {
		/* Assert RTS and DTR lines */
		save_flags(flags); cli();
		c->regs[5] |= DTR | RTS;
		c->pendregs[5] |= DTR | RTS;
		write_zsreg(c, R5, c->regs[5]);
		restore_flags(flags);
	}
}

/*
 * ------------------------------------------------------------
 * rs_ioctl() and friends
 * ------------------------------------------------------------
 */

static int z8530_get_serial_info(struct z8530_channel *c,
			   struct serial_struct * retinfo)
{
	struct serial_struct tmp;
  
	if (!retinfo)
		return -EFAULT;
	memset(&tmp, 0, sizeof(tmp));
	tmp.type = c->dev->type;
	tmp.line = c->line;
	tmp.port = -1; // c->port;
	tmp.irq = c->irq;
	tmp.flags = c->flags;
	tmp.baud_base = c->baud_base;
	tmp.close_delay = c->close_delay;
	tmp.closing_wait = c->closing_wait;
	tmp.custom_divisor = c->custom_divisor;
	return copy_to_user(retinfo,&tmp,sizeof(*retinfo));
}

static int z8530_set_serial_info(struct z8530_channel *c,
			   struct serial_struct * new_info)
{
	struct serial_struct new_serial;
	int retval = 0;

	if (!new_info)
		return -EFAULT;
	copy_from_user(&new_serial,new_info,sizeof(new_serial));
	if (!capable(CAP_SYS_ADMIN)) {
		if ((new_serial.baud_base != c->baud_base) ||
		    (new_serial.type != c->dev->type) ||
		    (new_serial.close_delay != c->close_delay) ||
		    ((new_serial.flags & ~ZILOG_USR_MASK) !=
		     (c->flags & ~ZILOG_USR_MASK)))
			return -EPERM;
		c->flags = ((c->flags & ~ZILOG_USR_MASK) |
			       (new_serial.flags & ZILOG_USR_MASK));
		c->custom_divisor = new_serial.custom_divisor;
		goto check_and_exit;
	}

	if (c->fdcount > 1)
		return -EBUSY;

	/*
	 * OK, past this point, all the error checking has been done.
	 * At this point, we start making changes.....
	 */

	c->baud_base = new_serial.baud_base;
	c->flags = ((c->flags & ~ZILOG_FLAGS) |
			(new_serial.flags & ZILOG_FLAGS));
	c->dev->type = new_serial.type;
	c->close_delay = new_serial.close_delay;
	c->closing_wait = new_serial.closing_wait;

check_and_exit:
	retval = z8530_async_startup(c);
	return retval;
}

/*
 * z8530_get_lsr_info - get line status register info
 *
 * Purpose: Let user call ioctl() to get info when the UART physically
 * 	    is emptied.  On bus types like RS485, the transmitter must
 * 	    release the bus after transmitting. This must be done when
 * 	    the transmit shift register is empty, not be done when the
 * 	    transmit holding register is empty.  This functionality
 * 	    allows an RS485 driver to be written in user space. 
 */

static int z8530_get_lsr_info(struct z8530_channel *c, unsigned int *value)
{
	unsigned char status;

	cli();
	status = read_zsreg(c, R0);
	sti();
	put_user(status,value);
	return 0;
}

static int z8530_get_modem_info(struct z8530_channel *c, unsigned int *value)
{
	unsigned char control, status;
	unsigned int result;

	cli();
	control = c->regs[5];
	status = read_zsreg(c, R0);
	sti();
	result =  ((control & RTS) ? TIOCM_RTS: 0)
		| ((control & DTR) ? TIOCM_DTR: 0)
		| ((status  & DCD) ? TIOCM_CAR: 0)
		| ((status  & CTS) ? 0: TIOCM_CTS);
	put_user(result,value);
	return 0;
}

static int z8530_set_modem_info(struct z8530_channel *c, unsigned int cmd,
			  unsigned int *value)
{
	int error;
	unsigned int arg, bits;

	error = verify_area(VERIFY_READ, value, sizeof(int));
	if (error)
		return error;
	get_user(arg, value);
	bits = (arg & TIOCM_RTS? RTS: 0) + (arg & TIOCM_DTR? DTR: 0);
	cli();
	switch (cmd) {
	case TIOCMBIS:
		c->regs[5] |= bits;
		break;
	case TIOCMBIC:
		c->regs[5] &= ~bits;
		break;
	case TIOCMSET:
		c->regs[5] = (c->regs[5] & ~(DTR | RTS)) | bits;
		break;
	default:
		sti();
		return -EINVAL;
	}
	c->pendregs[5] = c->regs[5];
	write_zsreg(c, R5, c->regs[5]);
	sti();
	return 0;
}

/*
 * rs_break - turn transmit break condition on/off
 */
static void z8530_break(struct tty_struct *tty, int break_state)
{
	struct z8530_channel *c = (struct z8530_channel *) tty->driver_data;
	unsigned long flags;

	if (serial_paranoia_check(c, tty->device, "rs_break"))
		return;
//	if (!c->port)
//		return;

	save_flags(flags); cli();
	if (break_state == -1)
		c->regs[5] |= SND_BRK;
	else
		c->regs[5] &= ~SND_BRK;
	write_zsreg(c, R5, c->regs[5]);
	restore_flags(flags);
}

static int z8530_async_ioctl(struct tty_struct *tty, struct file * file,
		    unsigned int cmd, unsigned long arg)
{
	int error;
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;

	if (serial_paranoia_check(c, tty->device, "rs_ioctl"))
		return -ENODEV;

	if ((cmd != TIOCGSERIAL) && (cmd != TIOCSSERIAL) &&
	    (cmd != TIOCSERCONFIG) && (cmd != TIOCSERGWILD)  &&
	    (cmd != TIOCSERSWILD) && (cmd != TIOCSERGSTRUCT)) {
		if (tty->flags & (1 << TTY_IO_ERROR))
		    return -EIO;
	}
	
	switch (cmd) {
		case TIOCMGET:
			error = verify_area(VERIFY_WRITE, (void *) arg,
				sizeof(unsigned int));
			if (error)
				return error;
			return z8530_get_modem_info(c, (unsigned int *) arg);
		case TIOCMBIS:
		case TIOCMBIC:
		case TIOCMSET:
			return z8530_set_modem_info(c, cmd, (unsigned int *) arg);
		case TIOCGSERIAL:
			error = verify_area(VERIFY_WRITE, (void *) arg,
						sizeof(struct serial_struct));
			if (error)
				return error;
			return z8530_get_serial_info(c,
					       (struct serial_struct *) arg);
		case TIOCSSERIAL:
			return z8530_set_serial_info(c,
					       (struct serial_struct *) arg);
		case TIOCSERGETLSR: /* Get line status register */
			error = verify_area(VERIFY_WRITE, (void *) arg,
				sizeof(unsigned int));
			if (error)
				return error;
			else
			    return z8530_get_lsr_info(c, (unsigned int *) arg);

#ifdef FIXME
		case TIOCSERGSTRUCT:
			error = verify_area(VERIFY_WRITE, (void *) arg,
						sizeof(struct z8530_serial));
			if (error)
				return error;
			copy_from_user((struct z8530_serial *) arg,
				       c, sizeof(struct z8530_serial));
			return 0;
#endif			
		default:
			return -ENOIOCTLCMD;
		}
	return 0;
}

static void z8530_set_termios(struct tty_struct *tty, struct termios *old_termios)
{
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;
	int was_stopped;

	if (tty->termios->c_cflag == old_termios->c_cflag)
		return;
	was_stopped = c->tx_stopped;

	change_speed(c);

	if (was_stopped && !c->tx_stopped)
		rs_start(tty);
}

/*
 * ------------------------------------------------------------
 * z8530_async_close()
 * 
 * This routine is called when the serial port gets closed.
 * Wait for the last remaining data to be sent.
 * ------------------------------------------------------------
 */
static void z8530_async_close(struct tty_struct *tty, struct file * filp)
{
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;
	unsigned long flags;

	if (!c || !c || serial_paranoia_check(c, tty->device, "rs_close"))
		return;
	
	save_flags(flags); cli();
	
	if (tty_hung_up_p(filp)) {
		restore_flags(flags);
		return;
	}
	
#ifdef SERIAL_DEBUG_OPEN
	printk("z8530_async_close ttys%d, count = %d\n", c->line, c->fdcount);
#endif
	if ((tty->count == 1) && (c->fdcount != 1)) {
		/*
		 * Uh, oh.  tty->count is 1, which means that the tty
		 * structure will be freed.  c->fdcount should always
		 * be one in these conditions.  If it's greater than
		 * one, we've got real problems, since it means the
		 * serial port won't be shutdown.
		 */
		printk("z8530_async_close: bad serial port count; tty->count is 1, "
		       "c->fdcount is %d\n", c->fdcount);
		c->fdcount = 1;
	}
	if (--c->fdcount < 0) {
		printk("z8530_async_close: bad serial port count for ttys%d: %d\n",
		       c->line, c->fdcount);
		c->fdcount = 0;
	}
	if (c->fdcount) {
		restore_flags(flags);
		return;
	}
	c->flags |= ZILOG_CLOSING;
	/*
	 * Save the termios structure, since this port may have
	 * separate termios for callout and dialin.
	 */
	if (c->flags & ZILOG_NORMAL_ACTIVE)
		c->normal_termios = *tty->termios;
	if (c->flags & ZILOG_CALLOUT_ACTIVE)
		c->callout_termios = *tty->termios;
	/*
	 * Now we wait for the transmit buffer to clear; and we notify 
	 * the line discipline to only process XON/XOFF characters.
	 */
	c->tty->closing = 1;
	if (c->closing_wait != ZILOG_CLOSING_WAIT_NONE)
		tty_wait_until_sent(tty, c->closing_wait);
	/*
	 * At this point we stop accepting input.  To do this, we
	 * disable the receiver and receive interrupts.
	 */
	c->regs[3] &= ~RxENABLE;
	c->pendregs[3] = c->regs[3];
	write_zsreg(c, R3, c->regs[3]);
	c->regs[1] &= ~(0x18);	/* disable any rx ints */
	c->pendregs[1] = c->regs[1];
	write_zsreg(c, R1, c->regs[1]);
	Z8530_flush_fifo(c);
	if (c->flags & ZILOG_INITIALIZED) {
		/*
		 * Before we drop DTR, make sure the SCC transmitter
		 * has completely drained.
		 */
		z8530_async_wait_until_sent(tty, c->timeout);
	}

	z8530_async_shutdown(c);
	if (tty->driver.flush_buffer)
		tty->driver.flush_buffer(tty);
	if (tty->ldisc.flush_buffer)
		tty->ldisc.flush_buffer(tty);
	tty->closing = 0;
	c->event = 0;
	c->tty = 0;
	if (c->blocked_open) {
		if (c->close_delay) {
			current->state = TASK_INTERRUPTIBLE;
			current->timeout = jiffies + c->close_delay;
			schedule();
		}
		wake_up_interruptible(&c->open_wait);
	}
	c->flags &= ~(ZILOG_NORMAL_ACTIVE|ZILOG_CALLOUT_ACTIVE|
			 ZILOG_CLOSING);
	wake_up_interruptible(&c->close_wait);
	restore_flags(flags);
}

/*
 * rs_wait_until_sent() --- wait until the transmitter is empty
 */

static void z8530_async_wait_until_sent(struct tty_struct *tty, int timeout)
{
	struct z8530_channel *c = (struct z8530_channel *) tty->driver_data;
	unsigned long orig_jiffies, char_time;

	if (serial_paranoia_check(c, tty->device, "z8530_async_wait_until_sent"))
		return;

	orig_jiffies = jiffies;
	/*
	 * Set the check interval to be 1/5 of the estimated time to
	 * send a single character, and make it at least 1.  The check
	 * interval should also be less than the timeout.
	 */
	char_time = (c->timeout - HZ/50) / c->xmit_fifo_size;
	char_time = char_time / 5;
	if (char_time == 0)
		char_time = 1;
	if (timeout)
		char_time = MIN(char_time, timeout);
	while ((read_zsreg(c, R1) & ALL_SNT) == 0) {
		current->state = TASK_INTERRUPTIBLE;
		current->counter = 0;	/* make us low-priority */
		current->timeout = jiffies + char_time;
		schedule();
		if (signal_pending(current))
			break;
		if (timeout && ((orig_jiffies + timeout) < jiffies))
			break;
	}
	current->state = TASK_RUNNING;
}

/*
 * z8530_async_hangup() --- called by tty_hangup() when a hangup is signaled.
 */
static void z8530_async_hangup(struct tty_struct *tty)
{
	struct z8530_channel *c = (struct z8530_channel *)tty->driver_data;

	if (serial_paranoia_check(c, tty->device, "z8530_async_hangup"))
		return;

	rs_flush_buffer(tty);
	z8530_async_shutdown(c);
	c->event = 0;
	c->fdcount = 0;
	c->flags &= ~(ZILOG_NORMAL_ACTIVE|ZILOG_CALLOUT_ACTIVE);
	c->tty = 0;
	wake_up_interruptible(&c->open_wait);
}

/*
 * ------------------------------------------------------------
 * z8530_async_open() and friends
 * ------------------------------------------------------------
 */
static int z8530_async_block_til_ready(struct tty_struct *tty, struct file * filp,
			   struct z8530_channel *c)
{
	struct wait_queue wait = { current, NULL };
	int		retval;
	int		do_clocal = 0;

	/*
	 * If the device is in the middle of being closed, then block
	 * until it's done, and then try again.
	 */
	if (c->flags & ZILOG_CLOSING) {
		interruptible_sleep_on(&c->close_wait);
#ifdef SERIAL_DO_RESTART
		return ((c->flags & ZILOG_HUP_NOTIFY) ?
			-EAGAIN : -ERESTARTSYS);
#else
		return -EAGAIN;
#endif
	}

	/*
	 * If this is a callout device, then just make sure the normal
	 * device isn't being used.
	 */
	if (tty->driver.subtype == SERIAL_TYPE_CALLOUT) {
		if (c->flags & ZILOG_NORMAL_ACTIVE)
			return -EBUSY;
		if ((c->flags & ZILOG_CALLOUT_ACTIVE) &&
		    (c->flags & ZILOG_SESSION_LOCKOUT) &&
		    (c->session != current->session))
		    return -EBUSY;
		if ((c->flags & ZILOG_CALLOUT_ACTIVE) &&
		    (c->flags & ZILOG_PGRP_LOCKOUT) &&
		    (c->pgrp != current->pgrp))
		    return -EBUSY;
		c->flags |= ZILOG_CALLOUT_ACTIVE;
		return 0;
	}
	
	/*
	 * If non-blocking mode is set, or the port is not enabled,
	 * then make the check up front and then exit.
	 */
	if ((filp->f_flags & O_NONBLOCK) ||
	    (tty->flags & (1 << TTY_IO_ERROR))) {
		if (c->flags & ZILOG_CALLOUT_ACTIVE)
			return -EBUSY;
		c->flags |= ZILOG_NORMAL_ACTIVE;
		return 0;
	}

	if (c->flags & ZILOG_CALLOUT_ACTIVE) {
		if (c->normal_termios.c_cflag & CLOCAL)
			do_clocal = 1;
	} else {
		if (tty->termios->c_cflag & CLOCAL)
			do_clocal = 1;
	}
	
	/*
	 * Block waiting for the carrier detect and the line to become
	 * free (i.e., not in use by the callout).  While we are in
	 * this loop, c->fdcount is dropped by one, so that
	 * rs_close() knows when to free things.  We restore it upon
	 * exit, either normal or abnormal.
	 */
	retval = 0;
	add_wait_queue(&c->open_wait, &wait);
#ifdef SERIAL_DEBUG_OPEN
	printk("block_til_ready before block: ttys%d, count = %d\n",
	       c->line, c->fdcount);
#endif
	cli();
	if (!tty_hung_up_p(filp)) 
		c->fdcount--;
	sti();
	c->blocked_open++;
	while (1) {
		cli();
		if (!(c->flags & ZILOG_CALLOUT_ACTIVE) &&
		    (tty->termios->c_cflag & CBAUD))
			z8530_rtsdtr(c, 1);
		sti();
		current->state = TASK_INTERRUPTIBLE;
		if (tty_hung_up_p(filp) ||
		    !(c->flags & ZILOG_INITIALIZED)) {
#ifdef SERIAL_DO_RESTART
			if (c->flags & ZILOG_HUP_NOTIFY)
				retval = -EAGAIN;
			else
				retval = -ERESTARTSYS;	
#else
			retval = -EAGAIN;
#endif
			break;
		}
		if (!(c->flags & ZILOG_CALLOUT_ACTIVE) &&
		    !(c->flags & ZILOG_CLOSING) &&
		    (do_clocal || (read_zsreg(c, R0) & DCD)))
			break;
		if (signal_pending(current)) {
			retval = -ERESTARTSYS;
			break;
		}
#ifdef SERIAL_DEBUG_OPEN
		printk("block_til_ready blocking: ttys%d, count = %d\n",
		       c->line, c->fdcount);
#endif
		schedule();
	}
	current->state = TASK_RUNNING;
	remove_wait_queue(&c->open_wait, &wait);
	if (!tty_hung_up_p(filp))
		c->fdcount++;
	c->blocked_open--;
#ifdef SERIAL_DEBUG_OPEN
	printk("block_til_ready after blocking: ttys%d, count = %d\n",
	       c->line, c->fdcount);
#endif
	if (retval)
		return retval;
	c->flags |= ZILOG_NORMAL_ACTIVE;
	return 0;
}	

/*
 * This routine is called whenever a serial port is opened.  It
 * enables interrupts for a serial port, linking in its ZILOG structure into
 * the IRQ chain.   It also performs the serial-specific
 * initialization for the tty structure.
 */

static int z8530_async_open(struct tty_struct *tty, struct file * filp)
{
	struct z8530_channel	*c;
	int 			retval, line;

	line = MINOR(tty->device) - tty->driver.minor_start;
	if ((line < 0) || (line >= z8530_channels_found))
		return -ENODEV;
	c = &z8530_channel_list[line];

	if (serial_paranoia_check(c, tty->device, "rs_open"))
		return -ENODEV;
#ifdef SERIAL_DEBUG_OPEN
	printk("rs_open %s%d, count = %d\n", tty->driver.name, c->line,
	       c->fdcount);
#endif

	c->fdcount++;
	tty->driver_data = c;
	c->tty = tty;

	/*
	 * If the port is the middle of closing, bail out now
	 */
	if (tty_hung_up_p(filp) ||
	    (c->flags & ZILOG_CLOSING)) {
		if (c->flags & ZILOG_CLOSING)
			interruptible_sleep_on(&c->close_wait);
#ifdef SERIAL_DO_RESTART
		return ((c->flags & ZILOG_HUP_NOTIFY) ?
			-EAGAIN : -ERESTARTSYS);
#else
		return -EAGAIN;
#endif
	}

	/*
	 * Start up serial port
	 */
	retval = z8530_async_startup(c);
	if (retval)
		return retval;

	retval = z8530_async_block_til_ready(tty, filp, c);
	if (retval) {
#ifdef SERIAL_DEBUG_OPEN
		printk("z8530_async_open returning after block_til_ready with %d\n",
		       retval);
#endif
		return retval;
	}

	if ((c->fdcount == 1) && (c->flags & ZILOG_SPLIT_TERMIOS)) {
		if (tty->driver.subtype == SERIAL_TYPE_NORMAL)
			*tty->termios = c->normal_termios;
		else 
			*tty->termios = c->callout_termios;
		z8530_change_speed(c);
	}

	c->session = current->session;
	c->pgrp = current->pgrp;

#ifdef SERIAL_DEBUG_OPEN
	printk("z8530_async_open ttys%d successful...", c->line);
#endif
	return 0;
}

/******************************* END ASYNC SUPPORT **************************/
#endif

/*
 *	Describe a Z8530 in a standard format. We must pass the I/O as
 *	the port offset isnt predictable. The main reason for this function
 *	is to try and get a common format of report.
 */

static char *z8530_type_name[]={
	"Z8530",
	"Z85C30",
	"Z85230"
};

void z8530_describe(struct z8530_dev *dev, char *mapping, int io)
{
	printk(KERN_INFO "%s: %s found at %s 0x%X, IRQ %d.\n",
		dev->name, 
		z8530_type_name[dev->type],
		mapping,
		Z8530_PORT_OF(io),
		dev->irq);
}

/*
 *	Configure up a Z8530
 */

 
int z8530_init(struct z8530_dev *dev)
{
	/* NOP the interrupt handlers first - we might get a
	   floating IRQ transition when we reset the chip */
	dev->chanA.irqs=&z8530_nop;
	dev->chanB.irqs=&z8530_nop;
	/* Reset the chip */
	write_zsreg(&dev->chanA, R9, 0xC0);
	udelay(200);
	/* Now check its valid */
	write_zsreg(&dev->chanA, R12, 0xAA);
	if(read_zsreg(&dev->chanA, R12)!=0xAA)
	{
		return -ENODEV;
	}
	write_zsreg(&dev->chanA, R12, 0x55);
	if(read_zsreg(&dev->chanA, R12)!=0x55)
		return -ENODEV;
		
	dev->type=Z8530;
	
	/*
	 *	See the application note.
	 */
	 
	write_zsreg(&dev->chanA, R15, 0x01);
	
	/*
	 *	If we can set the low bit of R15 then
	 *	the chip is enhanced.
	 */
	 
	if(read_zsreg(&dev->chanA, R15)==0x01)
	{
		/* This C30 versus 230 detect is from Klaus Kudielka's dmascc */
		/* Put a char in the fifo */
		write_zsreg(&dev->chanA, R8, 0);
		if(read_zsreg(&dev->chanA, R0)&Tx_BUF_EMP)
			dev->type = Z85230;	/* Has a FIFO */
		else
			dev->type = Z85C30;	/* Z85C30, 1 byte FIFO */
	}
		
	/*
	 *	The code assumes R7' and friends are
	 *	off. Use write_zsext() for these and keep
	 *	this bit clear.
	 */
	 
	write_zsreg(&dev->chanA, R15, 0);
		
	/*
	 *	At this point it looks like the chip is behaving
	 */
	 
	memcpy(dev->chanA.regs, reg_init, 16);
	memcpy(dev->chanB.regs, reg_init ,16);
	
	return 0;
}


int z8530_shutdown(struct z8530_dev *dev)
{
	/* Reset the chip */
	dev->chanA.irqs=&z8530_nop;
	dev->chanB.irqs=&z8530_nop;
	write_zsreg(&dev->chanA, R9, 0xC0);
	udelay(100);
	return 0;
}

/*
 *	Load a Z8530 channel up from the system data
 *	We use +16 to indicate the 'prime' registers
 */

 
int z8530_channel_load(struct z8530_channel *c, u8 *rtable)
{
	while(*rtable!=255)
	{
		int reg=*rtable++;
		if(reg>0x0F)
			write_zsreg(c, R15, c->regs[15]|1);
		write_zsreg(c, reg&0x0F, *rtable);
		if(reg>0x0F)
			write_zsreg(c, R15, c->regs[15]&~1);
		c->regs[reg]=*rtable++;
	}
	c->rx_function=z8530_null_rx;
	c->skb=NULL;
	c->tx_skb=NULL;
	c->tx_next_skb=NULL;
	c->mtu=1500;
	c->max=0;
	c->count=0;
	c->status=0;	/* Fixme - check DCD now */
	c->sync=1;
	write_zsreg(c, R3, c->regs[R3]|RxENABLE);
	return 0;
}


/*
 *	Higher level shovelling - transmit chains
 */

static void z8530_tx_begin(struct z8530_channel *c)
{
	unsigned long flags;
	if(c->tx_skb)
		return;
		
	c->tx_skb=c->tx_next_skb;
	c->tx_next_skb=NULL;
	c->tx_ptr=c->tx_next_ptr;
	
	mark_bh(NET_BH);
	if(c->tx_skb==NULL)
	{
		/* Idle on */
		if(c->dma_tx)
			disable_dma(c->txdma);
		c->txcount=0;
	}
	else
	{
		c->txcount=c->tx_skb->len;
		
		if(c->dma_tx)
		{
			/*
			 *	DMA mode transfers.
			 */
			 
			save_flags(flags);
			cli();
			disable_dma(c->txdma);

			/*
			 *	These two are needed by the 8530/85C30
			 *	and must be issued when idling.
			 */
			 
			if(c->dev->type!=Z85230)
			{
				write_zsctrl(c, RES_Tx_CRC);
				write_zsctrl(c, RES_EOM_L);
			}	
			write_zsreg(c, R10, c->regs[10]&~ABUNDER);

			/*
			 *	Then set up DMA
			 */

			clear_dma_ff(c->txdma);
			set_dma_addr(c->txdma, virt_to_bus(c->tx_ptr));
			set_dma_count(c->txdma, c->txcount);
			write_zsreg(c, R5, c->regs[R5]|TxENAB);
#if 0			
			{
				int i;
				printk("<:");
				for(i=0;i<c->txcount;i++)
					printk("%02X:", c->tx_ptr[i]);
				printk(">\n");
			}
#endif			
			enable_dma(c->txdma);
			restore_flags(flags);
		}
		else
		{
			save_flags(flags);
			cli();
			/* ABUNDER off */
			write_zsreg(c, R10, c->regs[10]);
			write_zsctrl(c, RES_Tx_CRC);
			write_zsctrl(c, RES_EOM_L);
	
			while(c->txcount && (read_zsreg(c,R0)&Tx_BUF_EMP))
			{		
				write_zsreg(c, R8, *c->tx_ptr++);
				c->txcount--;
			}
			restore_flags(flags);
		}
	}
}
 
 
static void z8530_tx_done(struct z8530_channel *c)
{
	unsigned long flags;
	struct sk_buff *skb;

	save_flags(flags);
	cli();

	c->netdevice->tbusy=0;
	mark_bh(NET_BH);
	/* Can't happen */
	if(c->tx_skb==NULL)
	{
		restore_flags(flags);
		return;
	}
	skb=c->tx_skb;
	c->tx_skb=NULL;
	z8530_tx_begin(c);
	restore_flags(flags);
	dev_kfree_skb(skb, FREE_WRITE);
}

/*
 *	Higher level shovelling - receive chains
 */
 
void z8530_null_rx(struct z8530_channel *c, struct sk_buff *skb)
{
	skb->free=1;
	kfree_skb(skb, FREE_READ);
}

static void z8530_rx_done(struct z8530_channel *c)
{
	struct sk_buff *skb;
	int ct;
	
	/*
	 *	Is our receive engine in DMA mode
	 */
	 
	if(c->rxdma_on)
	{
		/*
		 *	Save the ready state and the buffer currently
		 *	being used as the DMA target
		 */

		int ready=c->dma_ready;
		unsigned char *rxb=c->rx_buf[c->dma_num];
		
		/*
		 *	Complete this DMA. Neccessary to find the length
		 */	
		 
			
		disable_dma(c->rxdma);

		clear_dma_ff(c->rxdma);
		c->rxdma_on=0;
		ct=c->mtu-get_dma_residue(c->rxdma);
		if(ct<0)
			ct=2;	/* Shit happens.. */
		c->dma_ready=0;
		
		/*
		 *	Normal case: the other slot is free, start the next DMA
		 *	into it immediately.
		 */
		 
		if(ready)
		{
			unsigned long flags;
			save_flags(flags);
			cli();
			c->dma_num^=1;
			set_dma_mode(c->rxdma, DMA_MODE_READ|0x10);
			set_dma_addr(c->rxdma, virt_to_bus(c->rx_buf[c->dma_num]));
			set_dma_count(c->rxdma, c->mtu);
			c->rxdma_on = 1;
			enable_dma(c->rxdma);
			restore_flags(flags);
			/* Stop any frames that we missed the head of 
			   from passing */
			write_zsreg(c, R0, RES_Rx_CRC);
		}
		else
			/* Can't occur as we dont reenable the DMA irq until
			   after the flip is done */
			printk("DMA flip overrun!\n");
		
		/*
		 *	Shove the old buffer into an sk_buff. We can't DMA
		 *	directly into one on a PC - it might be above the 16Mb
		 *	boundary. Optimisation - we could check to see if we
		 *	can avoid the copy. Optimisation 2 - make the memcpy
		 *	a copychecksum.
		 */
		 
		skb=dev_alloc_skb(ct);
		if(skb==NULL)
			printk("%s: Memory squeeze.\n", c->netdevice->name);
		else
		{
			skb->free=1;
			skb_put(skb, ct);
			memcpy(skb->data, rxb, ct);
		}
		c->dma_ready=1;
	}
	else
	{
		RT_LOCK;	
		skb=c->skb;
		
		/*
		 *	The game we play for non DMA is similar. We want to
		 *	get the controller set up for the next packet as fast
		 *	as possible. We potentially only have one byte + the
		 *	fifo length for this. Thus we want to flip to the new
		 *	buffer and then mess around copying and allocating
		 *	things. For the current case it doesn't matter but
		 *	if you build a system where the sync irq isnt blocked
		 *	by the kernel IRQ disable then you need only block the
		 *	sync IRQ for the RT_LOCK area.
		 *	
		 */
		ct=c->count;
		
		c->skb = c->skb2;
		c->count = 0;
		c->max = c->mtu;
		if(c->skb)
		{
			c->dptr = c->skb->data;
			c->max = c->mtu;
		}
		else
		{
			c->count= 0;
			c->max = 0;
		}
		RT_UNLOCK;

		c->skb2 = dev_alloc_skb(c->mtu);
		if(c->skb2==NULL)
			printk(KERN_WARNING "%s: memory squeeze.\n",
				c->netdevice->name);
		else
		{
			c->skb2->free=1;
			skb_put(c->skb2,c->mtu);
		}
	}
	/*
	 *	If we received a frame we must now process it.
	 */
	if(skb)
	{
		skb_trim(skb, ct);
		c->rx_function(c,skb);
	}
	else
		printk("Lost a frame\n");
}

/*
 *	Cannot DMA over a 64K boundary
 */
 
extern inline int spans_boundary(struct sk_buff *skb)
{
	unsigned long a=(unsigned long)skb->data;
	a^=(a+skb->len);
	if(a&0x00010000)	/* Different 64K block ? */
		return 1;
	return 0;
}

/*
 *	Queue a packet for transmission. Because we have rather
 *	hard to hit interrupt latencies for the Z85230 per packet 
 *	even in DMA mode we do the flip to DMA buffer if needed here
 *	not in the IRQ.
 */

int z8530_queue_xmit(struct z8530_channel *c, struct sk_buff *skb)
{
	unsigned long flags;
	if(c->tx_next_skb)
	{
		skb->dev->tbusy=1;
		return 1;
	}
	
	/* PC SPECIFIC - DMA limits */
	
	/*
	 *	If we will DMA the transmit and its gone over the ISA bus
	 *	limit, then copy to the flip buffer
	 */

	if(c->dma_tx &&
		((unsigned long)(virt_to_bus(skb->data+skb->len))>=16*1024*1024
		|| spans_boundary(skb))
	)
	{
		/* 
		 *	Send the flip buffer, and flip the flippy bit.
		 *	We don't care which is used when just so long as
		 *	we never use the same buffer twice in a row. Since
		 *	only one buffer can be going out at a time the other
		 *	has to be safe.
		 */
		c->tx_next_ptr=c->tx_dma_buf[c->tx_dma_used];
		c->tx_dma_used^=1;	/* Flip temp buffer */
		memcpy(c->tx_next_ptr, skb->data, skb->len);
	}
	else
		c->tx_next_ptr=skb->data;	
	RT_LOCK;
	c->tx_next_skb=skb;
	RT_UNLOCK;
	
	save_flags(flags);
	cli();
	z8530_tx_begin(c);
	restore_flags(flags);
	return 0;
}


struct enet_statistics *z8530_get_stats(struct z8530_channel *c)
{
	return &c->stats;
}

#ifdef MODULE

/*
 *	Module support
 */
 
int init_module(void)
{
	printk(KERN_INFO "Generic Z85C30/Z85230 interface driver v0.2 BETA\n");
	return 0;
}

void cleanup_module(void)
{
}

#endif
